@@ -17,19 +17,18 @@ CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT
 OTHER DEALINGS IN THE SOFTWARE.
 
 '''
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import argparse
 import functools
 import glob
 import os
 import re
 import sys
+from typing import Any, Dict, Iterable, Iterator, List, MutableMapping, Optional, Sequence
 
 import setuptools
 
 from isort import SortImports, __version__
-from isort.settings import DEFAULT_SECTIONS, WrapModes, default, from_path, should_skip
+from isort.settings import DEFAULT_SECTIONS, WrapModes, default, file_should_be_skipped, from_path
 
 INTRO = r"""
 /#######################################################################\
@@ -57,7 +56,7 @@ INTRO = r"""
 shebang_re = re.compile(br'^#!.*\bpython[23w]?\b')
 
 
-def is_python_file(path):
+def is_python_file(path: str) -> bool:
     _root, ext = os.path.splitext(path)
     if ext in ('.py', '.pyi'):
         return True
@@ -78,12 +77,12 @@ def is_python_file(path):
 
 
 class SortAttempt(object):
-    def __init__(self, incorrectly_sorted, skipped):
+    def __init__(self, incorrectly_sorted: bool, skipped: bool) -> None:
         self.incorrectly_sorted = incorrectly_sorted
         self.skipped = skipped
 
 
-def sort_imports(file_name, **arguments):
+def sort_imports(file_name: str, **arguments: Any) -> Optional[SortAttempt]:
     try:
         result = SortImports(file_name, **arguments)
         return SortAttempt(result.incorrectly_sorted, result.skipped)
@@ -92,7 +91,7 @@ def sort_imports(file_name, **arguments):
         return None
 
 
-def iter_source_code(paths, config, skipped):
+def iter_source_code(paths: Iterable[str], config: MutableMapping[str, Any], skipped: List[str]) -> Iterator[str]:
     """Iterate over all Python source files defined in paths."""
     if 'not_skip' in config:
         config['skip'] = list(set(config['skip']).difference(config['not_skip']))
@@ -101,14 +100,14 @@ def iter_source_code(paths, config, skipped):
         if os.path.isdir(path):
             for dirpath, dirnames, filenames in os.walk(path, topdown=True, followlinks=True):
                 for dirname in list(dirnames):
-                    if should_skip(dirname, config, dirpath):
+                    if file_should_be_skipped(dirname, config, dirpath):
                         skipped.append(dirname)
                         dirnames.remove(dirname)
                 for filename in filenames:
                     filepath = os.path.join(dirpath, filename)
                     if is_python_file(filepath):
                         relative_file = os.path.relpath(filepath, path)
-                        if should_skip(relative_file, config, path):
+                        if file_should_be_skipped(relative_file, config, path):
                             skipped.append(filename)
                         else:
                             yield filepath
@@ -122,21 +121,21 @@ class ISortCommand(setuptools.Command):
     """
 
     description = "Run isort on modules registered in setuptools"
-    user_options = []
+    user_options = []  # type: List[Any]
 
-    def initialize_options(self):
+    def initialize_options(self) -> None:
         default_settings = default.copy()
         for key, value in default_settings.items():
             setattr(self, key, value)
 
-    def finalize_options(self):
+    def finalize_options(self) -> None:
         "Get options from config files."
-        self.arguments = {}
+        self.arguments = {}  # type: Dict[str, Any]
         computed_settings = from_path(os.getcwd())
         for key, value in computed_settings.items():
             self.arguments[key] = value
 
-    def distribution_files(self):
+    def distribution_files(self) -> Iterator[str]:
         """Find distribution packages."""
         # This is verbatim from flake8
         if self.distribution.packages:
@@ -155,7 +154,7 @@ class ISortCommand(setuptools.Command):
         # Don't miss the setup.py file itself
         yield "setup.py"
 
-    def run(self):
+    def run(self) -> None:
         arguments = self.arguments
         wrong_sorted_files = False
         arguments['check'] = True
@@ -171,7 +170,7 @@ class ISortCommand(setuptools.Command):
             sys.exit(1)
 
 
-def parse_args(argv=None):
+def parse_args(argv: Optional[Sequence[str]] = None) -> Dict[str, Any]:
     parser = argparse.ArgumentParser(description='Sort Python import definitions alphabetically '
                                                  'within logical sections. Run with no arguments to run '
                                                  'interactively. Run with `-` as the first argument to read from '
@@ -233,7 +232,7 @@ def parse_args(argv=None):
                         help="Forces line endings to the specified value. If not set, values will be guessed per-file.")
     parser.add_argument('-ls', '--length-sort', help='Sort imports by their string length.',
                         dest='length_sort', action='store_true')
-    parser.add_argument('-m', '--multi-line', dest='multi_line_output', type=int, choices=range(len(WrapModes)),
+    parser.add_argument('-m', '--multi-line', dest='multi_line_output', type=WrapModes.from_string,
                         help='Multi line output (0-grid, 1-vertical, 2-hanging, 3-vert-hanging, 4-vert-grid, '
                         '5-vert-grid-grouped, 6-vert-grid-grouped-no-comma).')
     inline_args_group.add_argument('-nis', '--no-inline-sort', dest='no_inline_sort', action='store_true',
@@ -296,6 +295,8 @@ def parse_args(argv=None):
                              'where it may not be safe to operate in')
     parser.add_argument('--case-sensitive', dest='case_sensitive', action='store_true',
                         help='Tells isort to include casing when sorting module names')
+    parser.add_argument('--filter-files', dest='filter_files', action='store_true',
+                        help='Tells isort to filter files even when they are explicitly passed in as part of the command')
     parser.add_argument('files', nargs='*', help='One or more Python source files that need their imports sorted.')
 
     arguments = {key: value for key, value in vars(parser.parse_args(argv)).items() if value}
@@ -306,7 +307,7 @@ def parse_args(argv=None):
     return arguments
 
 
-def main(argv=None):
+def main(argv: Optional[Sequence[str]] = None) -> None:
     arguments = parse_args(argv)
     if arguments.get('show_version'):
         print(INTRO)
@@ -332,23 +333,28 @@ def main(argv=None):
 
     file_names = arguments.pop('files', [])
     if file_names == ['-']:
-        try:
-            # python 3
-            file_ = sys.stdin.buffer
-        except AttributeError:
-            # python 2
-            file_ = sys.stdin
-        SortImports(file_=file_, write_to_stdout=True, **arguments)
+        SortImports(file_contents=sys.stdin.read(), write_to_stdout=True, **arguments)
     else:
         if not file_names:
             file_names = ['.']
             arguments['recursive'] = True
             if not arguments.get('apply', False):
                 arguments['ask_to_apply'] = True
-        config = from_path(os.path.abspath(file_names[0]) or os.getcwd()).copy()
+
+        config = from_path(arguments.get('settings_path', '') or os.path.abspath(file_names[0]) or os.getcwd()).copy()
         config.update(arguments)
         wrong_sorted_files = False
-        skipped = []
+        skipped = []  # type: List[str]
+
+        if config.get('filter_files'):
+            filtered_files = []
+            for file_name in file_names:
+                if file_should_be_skipped(file_name, config):
+                    skipped.append(file_name)
+                else:
+                    filtered_files.append(file_name)
+            file_names = filtered_files
+
         if arguments.get('recursive', False):
             file_names = iter_source_code(file_names, config, skipped)
         num_skipped = 0
@@ -361,7 +367,8 @@ def main(argv=None):
             executor = multiprocessing.Pool(jobs)
             attempt_iterator = executor.imap(functools.partial(sort_imports, **arguments), file_names)
         else:
-            attempt_iterator = (sort_imports(file_name, **arguments) for file_name in file_names)
+            # https://github.com/python/typeshed/pull/2814
+            attempt_iterator = (sort_imports(file_name, **arguments) for file_name in file_names)  # type: ignore
 
         for sort_attempt in attempt_iterator:
             if not sort_attempt:
