@@ -22,30 +22,24 @@ CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT
 OTHER DEALINGS IN THE SOFTWARE.
 
 """
-from __future__ import absolute_import, division, print_function, unicode_literals
-
+import configparser
+import enum
 import fnmatch
-import io
 import os
 import posixpath
 import re
-import sys
 import warnings
-from collections import namedtuple
 from distutils.util import strtobool
+from functools import lru_cache
+from pathlib import Path
+from typing import Any, Callable, Dict, Iterable, List, Mapping, MutableMapping, Union
 
-from .pie_slice import lru_cache
 from .utils import difference, union
 
-try:
-    import configparser
-except ImportError:
-    import ConfigParser as configparser
-
 try:
     import toml
 except ImportError:
-    toml = False
+    toml = None  # type: ignore
 
 try:
     import appdirs
@@ -62,9 +56,21 @@ safety_exclude_re = re.compile(
     r"|lib/python[0-9].[0-9]+)/"
 )
 
-WrapModes = ('GRID', 'VERTICAL', 'HANGING_INDENT', 'VERTICAL_HANGING_INDENT', 'VERTICAL_GRID', 'VERTICAL_GRID_GROUPED',
-             'VERTICAL_GRID_GROUPED_NO_COMMA', 'NOQA')
-WrapModes = namedtuple('WrapModes', WrapModes)(*range(len(WrapModes)))
+
+class WrapModes(enum.Enum):
+    GRID = 0  # 0
+    VERTICAL = 1
+    HANGING_INDENT = 2
+    VERTICAL_HANGING_INDENT = 3
+    VERTICAL_GRID = 4
+    VERTICAL_GRID_GROUPED = 5
+    VERTICAL_GRID_GROUPED_NO_COMMA = 6
+    NOQA = 7
+
+    @staticmethod
+    def from_string(value: str) -> 'WrapModes':
+        return getattr(WrapModes, str(value), None) or WrapModes(int(value))
+
 
 # Note that none of these lists must be complete as they are simply fallbacks for when included auto-detection fails.
 default = {'force_to_top': [],
@@ -117,7 +123,7 @@ default = {'force_to_top': [],
                                       'test', 'textwrap', 'this', 'thread', 'threading', 'time', 'timeit', 'tkinter',
                                       'token', 'tokenize', 'trace', 'traceback', 'tracemalloc', 'ttk', 'tty', 'turtle',
                                       'turtledemo', 'types', 'typing', 'unicodedata', 'unittest', 'urllib', 'urllib2',
-                                      'urlparse', 'user', 'usercustomize', 'uu', 'uuid', 'venv', 'videoreader',
+                                      'urlparse', 'usercustomize', 'uu', 'uuid', 'venv', 'videoreader',
                                       'warnings', 'wave', 'weakref', 'webbrowser', 'whichdb', 'winreg', 'winsound',
                                       'wsgiref', 'xdrlib', 'xml', 'xmlrpc', 'xmlrpclib', 'zipapp', 'zipfile',
                                       'zipimport', 'zlib'],
@@ -167,12 +173,15 @@ default = {'force_to_top': [],
 
 
 @lru_cache()
-def from_path(path):
+def from_path(path: Union[str, Path]) -> Dict[str, Any]:
     computed_settings = default.copy()
     isort_defaults = ['~/.isort.cfg']
     if appdirs:
         isort_defaults = [appdirs.user_config_dir('isort.cfg')] + isort_defaults
 
+    if isinstance(path, Path):
+        path = str(path)
+
     _update_settings_with_config(path, '.editorconfig', ['~/.editorconfig'], ('*', '*.py', '**.py'), computed_settings)
     _update_settings_with_config(path, 'pyproject.toml', [], ('tool.isort', ), computed_settings)
     _update_settings_with_config(path, '.isort.cfg', isort_defaults, ('settings', 'isort'), computed_settings)
@@ -181,7 +190,45 @@ def from_path(path):
     return computed_settings
 
 
-def _update_settings_with_config(path, name, default, sections, computed_settings):
+def prepare_config(settings_path: Path, **setting_overrides: Any) -> Dict[str, Any]:
+    config = from_path(settings_path).copy()
+    for key, value in setting_overrides.items():
+        access_key = key.replace('not_', '').lower()
+        # The sections config needs to retain order and can't be converted to a set.
+        if access_key != 'sections' and type(config.get(access_key)) in (list, tuple):
+            if key.startswith('not_'):
+                config[access_key] = list(set(config[access_key]).difference(value))
+            else:
+                config[access_key] = list(set(config[access_key]).union(value))
+        else:
+            config[key] = value
+
+    if config['force_alphabetical_sort']:
+        config.update({'force_alphabetical_sort_within_sections': True,
+                       'no_sections': True,
+                       'lines_between_types': 1,
+                       'from_first': True})
+
+    indent = str(config['indent'])
+    if indent.isdigit():
+        indent = " " * int(indent)
+    else:
+        indent = indent.strip("'").strip('"')
+        if indent.lower() == "tab":
+            indent = "\t"
+    config['indent'] = indent
+
+    config['comment_prefix'] = config['comment_prefix'].strip("'").strip('"')
+    return config
+
+
+def _update_settings_with_config(
+    path: str,
+    name: str,
+    default: Iterable[str],
+    sections: Iterable[str],
+    computed_settings: MutableMapping[str, Any]
+) -> None:
     editor_config_file = None
     for potential_settings_path in default:
         expanded = os.path.expanduser(potential_settings_path)
@@ -192,7 +239,7 @@ def _update_settings_with_config(path, name, default, sections, computed_setting
     tries = 0
     current_directory = path
     while current_directory and tries < MAX_CONFIG_SEARCH_DEPTH:
-        potential_path = os.path.join(current_directory, str(name))
+        potential_path = os.path.join(current_directory, name)
         if os.path.exists(potential_path):
             editor_config_file = potential_path
             break
@@ -207,7 +254,18 @@ def _update_settings_with_config(path, name, default, sections, computed_setting
         _update_with_config_file(editor_config_file, sections, computed_settings)
 
 
-def _update_with_config_file(file_path, sections, computed_settings):
+def _get_str_to_type_converter(setting_name: str) -> Callable[[str], Any]:
+    type_converter = type(default.get(setting_name, ''))  # type: Callable[[str], Any]
+    if type_converter == WrapModes:
+        type_converter = WrapModes.from_string
+    return type_converter
+
+
+def _update_with_config_file(
+    file_path: str,
+    sections: Iterable[str],
+    computed_settings: MutableMapping[str, Any]
+) -> None:
     cwd = os.path.dirname(file_path)
     settings = _get_config_data(file_path, sections).copy()
     if not settings:
@@ -230,7 +288,7 @@ def _update_with_config_file(file_path, sections, computed_settings):
 
     for key, value in settings.items():
         access_key = key.replace('not_', '').lower()
-        existing_value_type = type(default.get(access_key, ''))
+        existing_value_type = _get_str_to_type_converter(access_key)
         if existing_value_type in (list, tuple):
             # sections has fixed order values; no adding or substraction from any set
             if access_key == 'sections':
@@ -258,14 +316,21 @@ def _update_with_config_file(file_path, sections, computed_settings):
                 result = default.get(access_key) if value.lower().strip() == 'false' else 2
             computed_settings[access_key] = result
         else:
-            computed_settings[access_key] = existing_value_type(value)
+            computed_settings[access_key] = getattr(existing_value_type, str(value), None) or existing_value_type(value)
 
 
-def _as_list(value):
-    return filter(bool, [item.strip() for item in value.replace('\n', ',').split(',')])
+def _as_list(value: str) -> List[str]:
+    if isinstance(value, list):
+        return [item.strip() for item in value]
+    filtered = [
+        item.strip()
+        for item in value.replace('\n', ',').split(',')
+        if item.strip()
+    ]
+    return filtered
 
 
-def _abspaths(cwd, values):
+def _abspaths(cwd: str, values: Iterable[str]) -> List[str]:
     paths = [
         os.path.join(cwd, value)
         if not value.startswith(os.path.sep) and value.endswith(os.path.sep)
@@ -276,10 +341,10 @@ def _abspaths(cwd, values):
 
 
 @lru_cache()
-def _get_config_data(file_path, sections):
-    settings = {}
+def _get_config_data(file_path: str, sections: Iterable[str]) -> Dict[str, Any]:
+    settings = {}  # type: Dict[str, Any]
 
-    with io.open(file_path) as config_file:
+    with open(file_path) as config_file:
         if file_path.endswith('.toml'):
             if toml:
                 config = toml.load(config_file)
@@ -289,10 +354,10 @@ def _get_config_data(file_path, sections):
                         config_section = config_section.get(key, {})
                     settings.update(config_section)
             else:
-                warnings.warn(
-                    "Found %s but toml package is not installed. To configure "
-                    "isort with %s, install with 'isort[pyproject]'." % (file_path, file_path)
-                )
+                if '[tool.isort]' in config_file.read():
+                    warnings.warn("Found {} with [tool.isort] section, but toml package is not installed. "
+                                  "To configure isort with {}, install with 'isort[pyproject]'.".format(file_path,
+                                                                                                        file_path))
         else:
             if file_path.endswith('.editorconfig'):
                 line = '\n'
@@ -304,13 +369,8 @@ def _get_config_data(file_path, sections):
                         break
                     last_position = config_file.tell()
 
-            if sys.version_info >= (3, 2):
-                config = configparser.ConfigParser(strict=False)
-                config.read_file(config_file)
-            else:
-                config = configparser.SafeConfigParser()
-                config.readfp(config_file)
-
+            config = configparser.ConfigParser(strict=False)
+            config.read_file(config_file)
             for section in sections:
                 if config.has_section(section):
                     settings.update(config.items(section))
@@ -318,7 +378,11 @@ def _get_config_data(file_path, sections):
     return settings
 
 
-def should_skip(filename, config, path=''):
+def file_should_be_skipped(
+    filename: str,
+    config: Mapping[str, Any],
+    path: str = ''
+) -> bool:
     """Returns True if the file and/or folder should be skipped based on the passed in settings."""
     os_path = os.path.join(path, filename)
 
